'use strict';
const path = require('path');
const express = require('express');
const cookieParser = require('cookie-parser');
const { router: authRouter, ensureAuthenticated } = require('./lib/auth.js')
const templates = require('./lib/templates.js');
const email = require('./lib/email.js');
const gitlab = require('./lib/gitlab.js');
const loggers = require('./lib/logging.js').loggers;
const config = require('./lib/config.js');
const spamTrap = require('./lib/spam-trap.js')(config.spamTraps);
const linkify = require('./lib/linkify.js');

const submitlog = loggers.submit;
const maillog = loggers.mail;

const send = email(config.email.server);
const app = express();
const port = config.port;

var expressOptions = {
    dotfiles: 'ignore',
    etag: false,
    index: false,
    maxAge: '1d',
    redirect: false,
    setHeaders: function (res, path, stat) {
        res.set('x-timestamp', Date.now());
    }
};

app.set('trust proxy', true); // Assume we're behind a proxy
app.use(express.static('/', expressOptions));
app.use(express.json());
app.use(express.urlencoded({extended: true}));
app.use(cookieParser(config.cookieSecret, {
  maxAge: 60 * 60 * 24 * 30, // one month
  httpOnly: true,
  secure: true,
}));

app.use('/', express.static(path.join(__dirname, 'static')))

app.post('/', (req, res) => {
    const sendConfig = config.email.registration || {};
    const templateContext = {
        service: "social.coop",
        config: {theme: config.theme},
        request: req,
    };
    
    function ok() {}
    function error() {
        return (err) => submitlog.info(`$message: `, err);
    }

    try {
      req.body['oc-user'] = linkify(req.body['oc-user'], 'opencollective.com');
    } catch(err) {
      submitlog.info("no OpenCollective entered for: " + JSON.stringify(req.body)); 
    }
    
    // This function logs spamtrap hits, we just need to drop spam.
    // Previously we sent a false ack email to spam requests, but
    // this was causing problems with Mailgun! So, no more false acks.
    if (!spamTrap(req.body)) {
        if (req.body.email) {
            // Send acknowledgement email to registrant
            const acknowledgementConfig = {
                from: sendConfig.from,
                to: req.body.email,
                subject: sendConfig.acknowledgement,
            };
            try {
                const html = templates.acknowledgement(templateContext);
                submitlog.info(`emailing registration acknowledgement to ${acknowledgementConfig.to}: `);
                send(acknowledgementConfig,
                     html)
                    .then(ok, error("error emailing registration acknowledgement"));
            }
            catch(e) {
                maillog.error(`failed to send registration acknowledgement to ${acknowledgementConfig.to} `, e);
            }
        }
        else {
            maillog.error(`cannot send registration acknowledgement, no email set in registration`);
        }

        if (gitlab) {
            try {
                gitlab.newRegistrationIssue(templateContext)
                    .then(ok, error("error posting new gitlab issue"));
            }
            catch(e) {
                submitlog.info('error posting new gitlab issue: ', e);
            }
        }

        // Send notification of registration to admins
        submitlog.info(`emailing registration notification to ${sendConfig.to}`, req.body);
        const notificationConfig = {
            from: sendConfig.from,
            to: sendConfig.to,
            subject: sendConfig.notification,
        };
        try {
            const html = templates.notification(templateContext);
            send(notificationConfig,
                 html,
                 req.body)
                .then(ok, error("error emailing registration notification"));
        }
        catch(e) {
            maillog.error(`failed to send notification of registration to ${notificationConfig.to}: `, e);
        }
    }
    res.redirect('/thankyou.html');
});

// socialcoop.meet.coop registration form requires users to be authenticated
// with Mastodon at social.coop

app.use('/auth', authRouter);

app.all(/^\/meet.coop-registration-form(.html)?/, ensureAuthenticated, (_req, _res, next) => {
    next();
});

app.post('/meet.coop-registration-form', (req, res) => {
    const sendConfig = config.email.meetCoopRegistration || {};
    const templateContext = {
        service: "meet.coop",
        config: {theme: config.theme},
        request: req,
    };

    function ok() {}
    function error() {
        return (err) => submitlog.info(`$message: `, err);
    }

    req.body['oc-user'] = linkify(req.body['oc-user'], 'opencollective.com')
    req.body['mastodon'] = linkify(req.body['mastodon'], 'social.coop')

    if (!spamTrap(req.body)) {
        console.log("ok")
        if (req.body.email) {
            // Send acknowledgement email to registrant
            const acknowledgementConfig = {
                from: sendConfig.from,
                to: req.body.email,
                subject: sendConfig.acknowledgement,
            };
            try {
                const html = templates.meetCoopAcknowledgement(templateContext);
                submitlog.info(`emailing meet.coop registration acknowledgement to ${acknowledgementConfig.to}: `);
                send(acknowledgementConfig,
                     html)
                    .then(ok, error("error emailing meet.coop registration acknowledgement"));
            }
            catch(e) {
                maillog.error(`failed to send meet.coop registration acknowledgement to ${acknowledgementConfig.to} `, e);
            }
        }
        else {
            maillog.error(`cannot send meet.coop registration acknowledgement, no email set in registration`);
        }

        if (gitlab) {
            try {
                gitlab.newRegistrationIssue(templateContext)
                    .then(ok, error("error posting new gitlab issue"));
            }
            catch(e) {
                submitlog.info('error posting new gitlab issue: ', e);
            }
        }

        // Send notification of registration to admins
        submitlog.info(`emailing meet.coop registration notification to ${sendConfig.to}`, req.body);
        const notificationConfig = {
            from: sendConfig.from,
            to: sendConfig.to,
            subject: sendConfig.notification,
        };
        try {
            const html = templates.meetCoopNotification(templateContext);
            send(notificationConfig,
                 html,
                 req.body)
                .then(ok, error("error emailing meet.coop registration notification"));
        }
        catch(e) {
            maillog.error(`failed to send notification of meet.coop registration to ${notificationConfig.to}: `, e);
        }
    }
    res.redirect('/meet.coop-registration-thankyou.html');
});


app.listen(port, () => console.log(`serve.js listening on port ${port}!`));
